﻿using BuzzGUI.Common;
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Media;
using System.Windows.Resources;
using System.Windows.Threading;

namespace WDE.AudioBlock
{
    class AboutWindow : Window
    {
        private string audioBlockVersion;
        private string aboutText;

        private static string defaultAboutText = @"AudioBlock Version {0}
(C) 2020 WDE

LibSampleRate (C) Erik de Castro Lopo
Managed Wrapper for LibSampleRate (C) 2015 Mario Guggenberger
Spline code (C) Ryan Seghers
SoundTouch library Copyright (C) Olli Parviainen 2001-2019
Free code examples around the Web (C) Respected owners
(C) Robotman for Amiga cursor
(C) Alan Tinsley for 'Topaz' font

Big thanks to:
Oskari Tammelin for Jeskola Buzz
IXix, polac, UNZ and many other Buzz plugin developers.

...and Buzz community helping to improve AudioBlock!

...
            
And keep buzzing :)
                   
";

        private Border guruBorder;
        private TextBlock tbContent;

        private DispatcherTimer dispatcherTimer;
        private DispatcherTimer dispatcherContentText;
        private DispatcherTimer waitTimer;
        private int aboutTextTypingPos = 0;

        private Cursor cursor;

        public AboutWindow(string ver, string txt = "")
        {
            audioBlockVersion = ver;
            aboutText = txt;
            if (txt == "")
            {
                aboutText = String.Format(defaultAboutText, audioBlockVersion);
            }

            this.Width = 640;
            this.Height = 500;

            this.Background = Brushes.Black;

            this.WindowStyle = WindowStyle.None;
            WindowStartupLocation = WindowStartupLocation.CenterScreen;
            ResizeMode = ResizeMode.NoResize;

            WindowInteropHelper helper = new WindowInteropHelper(this);
            helper.Owner = Global.Buzz.MachineViewHWND;

            Grid aboutGrid = new Grid() { HorizontalAlignment = HorizontalAlignment.Stretch, Margin = new Thickness(6) };
            aboutGrid.RowDefinitions.Add(new RowDefinition() { Height = new GridLength(80) });
            aboutGrid.RowDefinitions.Add(new RowDefinition());

            guruBorder = new Border();
            guruBorder.BorderThickness = new Thickness(6);
            guruBorder.BorderBrush = Brushes.Red;

            TextBlock tbGuru = new TextBlock() { VerticalAlignment = VerticalAlignment.Center };
            tbGuru.Text = "Software Failure. Press left mouse button to continue.\n" +
                "Guru Meditation #00000004.000" + audioBlockVersion;
            tbGuru.Foreground = Brushes.Red;
            tbGuru.TextAlignment = TextAlignment.Center;
            tbGuru.FontFamily = new FontFamily(new Uri("pack://application:,,,/AudioBlock.NET;Component/Resources/"), "./#Topaz New");
            tbGuru.FontSize = 18;
            guruBorder.Child = tbGuru;

            Grid.SetRow(guruBorder, 0);
            aboutGrid.Children.Add(guruBorder);

            tbContent = new TextBlock() { HorizontalAlignment = HorizontalAlignment.Stretch, Margin = new Thickness(6, 20, 6, 20) };
            tbContent.Foreground = Brushes.Red;
            tbContent.TextAlignment = TextAlignment.Center;
            tbContent.FontFamily = new FontFamily(new Uri("pack://application:,,,/AudioBlock.NET;Component/Resources/"), "./#Topaz New");
            tbContent.FontSize = 18;

            Grid.SetRow(tbContent, 1);
            aboutGrid.Children.Add(tbContent);

            dispatcherContentText = new DispatcherTimer();
            dispatcherTimer = new DispatcherTimer();
            waitTimer = new DispatcherTimer();

            StreamResourceInfo sriCursor = Application.GetResourceStream(new Uri("pack://application:,,,/AudioBlock.NET;Component/Resources/pointer.cur"));
            cursor = new Cursor(sriCursor.Stream);

            this.Content = aboutGrid;
            this.MouseDown += AboutWindow_MouseDown;
            this.MouseEnter += AboutWindow_MouseEnter;

            this.Loaded += AboutWindow_Loaded;
            this.Closed += AboutWindow_Closed;
        }

        private void AboutWindow_Closed(object sender, EventArgs e)
        {
            dispatcherTimer.Stop();
            dispatcherContentText.Stop();
            waitTimer.Stop();
        }

        private void AboutWindow_Loaded(object sender, RoutedEventArgs e)
        {
            dispatcherTimer.Interval = new TimeSpan(0, 0, 0, 0, 800);
            dispatcherTimer.Tick += DispatcherTimer_Tick;
            dispatcherTimer.Start();

            // Wait 3 seconds before starting to type text

            waitTimer.Interval = new TimeSpan(0, 0, 0, 3, 0);
            waitTimer.Tick += WaitTimer_Tick;
            waitTimer.Start();
        }

        private void WaitTimer_Tick(object sender, EventArgs e)
        {
            waitTimer.Stop();
            dispatcherContentText = new DispatcherTimer();
            dispatcherContentText.Interval = new TimeSpan(0, 0, 0, 0, (int)(GetTypingDelay() * 999));
            dispatcherContentText.Tick += DispatcherContentText_Tick;
            dispatcherContentText.Start();
        }

        private void DispatcherContentText_Tick(object sender, EventArgs e)
        {
            aboutTextTypingPos++;

            if (aboutTextTypingPos >= defaultAboutText.Length)
            {
                aboutTextTypingPos = 0;
                StreamResourceInfo sriCursor = Application.GetResourceStream(new Uri("pack://application:,,,/AudioBlock.NET;Component/Resources/busy.cur"));
                cursor = new Cursor(sriCursor.Stream);
                this.Cursor = cursor;
            }

            tbContent.Text = aboutText.Substring(0, aboutTextTypingPos);

            dispatcherContentText.Interval = new TimeSpan(0, 0, 0, 0, (int)(GetTypingDelay() * 999));
            dispatcherContentText.Start();
        }

        private double GetTypingDelay()
        {
            Random rndDelay = new Random();
            return rndDelay.NextDouble() * 0.1 + 0.1;
        }

        private void DispatcherTimer_Tick(object sender, EventArgs e)
        {
            guruBorder.BorderBrush = guruBorder.BorderBrush == Brushes.Red ? Brushes.Black : Brushes.Red;
        }

        private void AboutWindow_MouseEnter(object sender, System.Windows.Input.MouseEventArgs e)
        {
            this.Cursor = cursor;
        }

        private void AboutWindow_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            this.Close();
        }
    }
}
