﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;

namespace WDE.AudioBlock
{
    class EnvelopeBox : Canvas
    {
        public IEnvelopeLayer EnveloperLayer { get; }

        private Rectangle rect;

        // Seconds from beginning from pattern
        private bool freezed;
        MenuItem miFreeze;

        public bool Freezed
        {
            get { return freezed; }
            set
            {
                freezed = value;
                miFreeze.IsChecked = freezed;
            }
        }

        public EnvelopeBox(IEnvelopeLayer enveloperL, double width, double height, Brush fill, Brush stroke)
        {
            this.Width = width;
            this.Height = height;

            this.EnveloperLayer = enveloperL;

            rect = new Rectangle();
            rect.Width = width;
            rect.Height = height;
            rect.Fill = fill;
            rect.Stroke = stroke;

            Canvas.SetLeft(rect, 0);
            Canvas.SetTop(rect, 0);
            this.Children.Add(rect);

            ContextMenu contextMenu = new ContextMenu() { Margin = new Thickness(4, 4, 4, 4) };
            miFreeze = new MenuItem();
            miFreeze.Header = "Freezed";
            miFreeze.IsCheckable = true;
            miFreeze.Checked += Mi_Checked;
            miFreeze.Unchecked += Mi_Unchecked;
            contextMenu.Items.Add(miFreeze);

            MenuItem mi = new MenuItem();
            mi.Header = "Reset";
            mi.Click += Mi_Click_Reset;
            contextMenu.Items.Add(mi);

            mi = new MenuItem();
            mi.Header = "Delete";
            mi.Click += Mi_Click_Delete;
            contextMenu.Items.Add(mi);

            this.CacheMode = new BitmapCache() { EnableClearType = false, SnapsToDevicePixels = false, RenderAtScale = 1 };

            this.ContextMenu = contextMenu;
            Freezed = true;
        }

        private void Mi_Click_Reset(object sender, RoutedEventArgs e)
        {
            if (!Freezed)
                this.EnveloperLayer.ResetEnvelopeBox(this);
        }

        private void Mi_Click_Delete(object sender, RoutedEventArgs e)
        {
            if (!Freezed)
                this.EnveloperLayer.DeleteEnvelopeBox(this);
        }

        private void Mi_Unchecked(object sender, RoutedEventArgs e)
        {
            this.Freezed = false;
        }

        private void Mi_Checked(object sender, RoutedEventArgs e)
        {
            this.Freezed = true;
        }

        protected override void OnRender(DrawingContext dc)
        {
            base.OnRender(dc);
        }
    }
}
