﻿using BuzzGUI.Common;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using static WDE.AudioBlock.AudioBlock;

namespace WDE.AudioBlock
{

    /// <summary>
    /// Pan and Volume envelopes are pretty much copies with small differences.
    /// </summary>
    class EnvelopeLayerPan : EnvelopeBase, IEnvelopeLayer
    {
        private Point newBoxPosition;
        private double ENVELOPE_MAX_PAN = 2.0;

        public EnvelopeLayerPan() : base()
        {
            strokeBrush = Brushes.Black;
            fillBrush = Brushes.Orange;
            lineBrush = new SolidColorBrush(Color.FromArgb(0x80, 0x30, 0x40, 0x30));
        }

        internal void LoadData()
        {
            if (audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].PanEnvelopePoints.Count == 0)
            {
                PanEnvelopePoint vep = new PanEnvelopePoint();
                vep.Value = this.DefaulBoxValue();
                vep.TimeStamp = 0.0; // First time stamp starts always from 0 seconds.
                audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].PanEnvelopePoints.Add(vep);

                vep = new PanEnvelopePoint();
                vep.Value = this.DefaulBoxValue();
                vep.TimeStamp = audioBlock.GetSampleLengthInSecondsWithOffset(audioBlockIndex); // Last time stamp is the lenght of sample.
                audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].PanEnvelopePoints.Add(vep);
            }
        }

        public void Draw()
        {
            this.Children.Clear();
            envelopeBoxes.Clear();
            this.Children.Add(envPolyLine);

            if (viewMode == ViewOrientationMode.Horizontal)
            {
                foreach (PanEnvelopePoint vep in audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].PanEnvelopePoints)
                {

                    EnvelopeBox env = new EnvelopeBox(this, 10, 10, fillBrush, strokeBrush);
                    envelopeBoxes.Add(env);
                    env.MouseLeftButtonDown += Evb_MouseLeftButtonDown;

                    Point point = new Point(ValueToScreen(vep.Value), ConvertTimeStampInSecondsToPixels(vep.TimeStamp));
                    point = UpdateEnvelopeBoxCenterPosition(env, point, false);

                    Canvas.SetLeft(env, point.X - env.Width / 2);
                    Canvas.SetTop(env, point.Y - env.Height / 2);

                    env.ToolTip = string.Format("Time: {0:0.0}, Pan: {1:0.0}", vep.TimeStamp, Decibel.FromAmplitude(vep.Value) - 1);

                    this.Children.Add(env);
                }
            }
            else
            {
                foreach (PanEnvelopePoint vep in audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].PanEnvelopePoints)
                {
                    EnvelopeBox env = new EnvelopeBox(this, 10, 10, fillBrush, strokeBrush);
                    envelopeBoxes.Add(env);
                    env.MouseLeftButtonDown += Evb_MouseLeftButtonDown;

                    Point point = new Point(ConvertTimeStampInSecondsToPixels(vep.TimeStamp), ValueToScreen(vep.Value));
                    point = UpdateEnvelopeBoxCenterPosition(env, point, false);

                    Canvas.SetLeft(env, point.X - env.Width / 2);
                    Canvas.SetTop(env, point.Y - env.Height / 2);

                    env.ToolTip = string.Format("Time: {0:0.0}, Pan: {1:0.0}", vep.TimeStamp, vep.Value - 1);

                    this.Children.Add(env);
                }
            }

            UpdatePolyLinePath();
        }

        public void UpdateToolTip(EnvelopeBox env)
        {
            env.ToolTip = string.Format("Time: {0:0.00} Seconds, Pan: {1:0.0}", GetEnvelopeTimeStamp(env), GetEnvelopeValue(env) - 1);
        }

        public void UpdatePolyLinePath()
        {
            envPolyLine.Points.Clear();
            envPolyLine.Stroke = lineBrush;
            envPolyLine.StrokeThickness = 2;

            if (viewMode == ViewOrientationMode.Horizontal)
            {
                if (!audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopeCurves)
                {
                    foreach (EnvelopeBox evb in envelopeBoxes)
                    {
                        Point point = new Point(ValueToScreen(GetEnvelopeValue(evb)), ConvertTimeStampInSecondsToPixels(GetEnvelopeTimeStamp(evb)));
                        point = UpdateEnvelopeBoxCenterPosition(evb, point, false);
                        envPolyLine.Points.Add(point);
                    }
                }
                else
                {
                    //audioBlock.UpdateSplinePan(audioBlockIndex);
                    SplineCache spline = audioBlock.GetSplinePan(audioBlockIndex);
                    double slidingOffsetWindowInSeconds = envelopesCanvas.HostCanvas.SlidingWindowOffsetSeconds;

                    for (int i = 0; i < spline.YValues.Length; i++)
                    {
                        double y = this.Height * (((double)i * spline.StepSizeInSeconds - slidingOffsetWindowInSeconds) / DrawLengthInSeconds);
                        double x = this.Width * ((spline.YValues[i] / MaxBoxValue()) * ENVELOPE_VIEW_SCALE_ADJUST + ((1.0 - ENVELOPE_VIEW_SCALE_ADJUST) / 2.0));
                        Point polyPoint = new Point(x, y);
                        envPolyLine.Points.Add(polyPoint);
                    }
                }
            }
            else
            {
                if (!audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopeCurves)
                {
                    foreach (EnvelopeBox evb in envelopeBoxes)
                    {
                        Point point = new Point(ConvertTimeStampInSecondsToPixels(GetEnvelopeTimeStamp(evb)), ValueToScreen(GetEnvelopeValue(evb)));
                        point = UpdateEnvelopeBoxCenterPosition(evb, point, false);
                        envPolyLine.Points.Add(point);

                    }
                }
                else
                {
                    SplineCache spline = audioBlock.GetSplinePan(audioBlockIndex);
                    double slidingOffsetWindowInSeconds = envelopesCanvas.HostCanvas.SlidingWindowOffsetSeconds;

                    for (int i = 0; i < spline.YValues.Length; i++)
                    {
                        double x = this.Width * (((double)i * spline.StepSizeInSeconds - slidingOffsetWindowInSeconds) / DrawLengthInSeconds);
                        double y = this.Height * ((1.0 - (spline.YValues[i] / MaxBoxValue())) * ENVELOPE_VIEW_SCALE_ADJUST + ((1.0 - ENVELOPE_VIEW_SCALE_ADJUST) / 2.0));
                        Point polyPoint = new Point(x, y);
                        envPolyLine.Points.Add(polyPoint);
                    }
                }
            }
        }


        public void DeleteEnvelopeBox(EnvelopeBox envelopeBox)
        {
            // Can't delete first or last
            int envelopeBoxIndex = envelopeBoxes.IndexOf(envelopeBox);
            if ((envelopeBoxIndex > 0) && (envelopeBoxIndex < envelopeBoxes.Count - 1))
            {
                envPolyLine.Points.RemoveAt(envelopeBoxIndex);
                envelopeBoxes.Remove(envelopeBox);
                this.Children.Remove(envelopeBox);

                audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopePoints.RemoveAt(envelopeBoxIndex);
                if (audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopeCurves)
                    audioBlock.UpdateSplinePan(audioBlockIndex);
                UpdatePolyLinePath();
            }

        }

        internal override double GetEnvelopeValue(EnvelopeBox envelopeBox)
        {
            int envelopeBoxIndex = envelopeBoxes.IndexOf(envelopeBox);
            return GetEnvelopeValue(envelopeBoxIndex);
        }

        internal override double GetEnvelopeValue(int envelopeBoxIndex)
        {
            return audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopePoints[envelopeBoxIndex].Value;
        }

        internal override double GetEnvelopeTimeStamp(EnvelopeBox envelopeBox)
        {
            int envelopeBoxIndex = envelopeBoxes.IndexOf(envelopeBox);
            return GetEnvelopeTimeStamp(envelopeBoxIndex);
        }

        internal override double GetEnvelopeTimeStamp(int envelopeBoxIndex)
        {
            return audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopePoints[envelopeBoxIndex].TimeStamp;
        }

        internal override void SetEnvelopeValue(EnvelopeBox envelopeBox, double value)
        {
            int envelopeBoxIndex = envelopeBoxes.IndexOf(envelopeBox);
            SetEnvelopeValue(envelopeBoxIndex, value);
        }

        internal override void SetEnvelopeValue(int envelopeBoxIndex, double value)
        {
            audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopePoints[envelopeBoxIndex].Value = value;
        }

        internal override void SetEnvelopeTimeStamp(EnvelopeBox envelopeBox, double value)
        {
            int envelopeBoxIndex = envelopeBoxes.IndexOf(envelopeBox);
            SetEnvelopeTimeStamp(envelopeBoxIndex, value);
        }

        internal override void SetEnvelopeTimeStamp(int envelopeBoxIndex, double value)
        {
            audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopePoints[envelopeBoxIndex].TimeStamp = value;
        }

        public new void ResetEnvelopeBox(EnvelopeBox envelopeBox)
        {
            base.ResetEnvelopeBox(envelopeBox);

            if (audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopeCurves)
                audioBlock.UpdateSplinePan(audioBlockIndex);
            this.UpdatePolyLinePath();
        }

        public new void Init(AudioBlock ab, Envelopes c)
        {
            base.Init(ab, c);

            envelopesCanvas.SizeChanged += EnvelopesCanvas_SizeChanged;

            envelopesCanvas.HostCanvas.MouseMove += C_MouseMove;
            envelopesCanvas.HostCanvas.MouseRightButtonDown += HostCanvas_MouseRightButtonDown;
            envelopesCanvas.HostCanvas.MouseLeave += HostCanvas_MouseLeave;
            envelopesCanvas.HostCanvas.MouseLeftButtonDown += HostCanvas_MouseLeftButtonDown;
            envelopesCanvas.HostCanvas.MouseLeftButtonUp += HostCanvas_MouseLeftButtonUp;

            LoadData();

            // Start always hidden
            EnvelopeVisible = false;
        }

        private void HostCanvas_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            this.boxDragged = false;
        }

        private void HostCanvas_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (Keyboard.IsKeyDown(Key.LeftCtrl) || Keyboard.IsKeyDown(Key.RightCtrl))
            {
                List<EEnvelopeType> visibleEnvelopes = envelopesCanvas.VisbleEnvelopes();
                if (visibleEnvelopes.Count == 1 && visibleEnvelopes[0] == EEnvelopeType.Pan)
                {
                    Add_Point(newBoxPosition);
                }
            }
        }

        private void EnvelopesCanvas_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            this.Width = ((Canvas)sender).Width;
            this.Height = ((Canvas)sender).Height;
            Draw();
        }

        internal override double DefaulBoxValue()
        {
            return ENVELOPE_MAX_PAN / 2.0;
        }

        internal override double MaxBoxValue()
        {
            return ENVELOPE_MAX_PAN;
        }

        private void HostCanvas_MouseLeave(object sender, MouseEventArgs e)
        {
            this.boxDragged = false;
        }

        private void HostCanvas_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            newBoxPositionFromMenu = e.GetPosition(envelopesCanvas.HostCanvas);
        }

        private void Evb_MouseLeftButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if ((Keyboard.IsKeyDown(Key.LeftShift) || Keyboard.IsKeyDown(Key.RightShift)) && !((EnvelopeBox)sender).Freezed)
            {
                List<EEnvelopeType> visibleEnvelopes = envelopesCanvas.VisbleEnvelopes();
                if (visibleEnvelopes.Count == 1 && visibleEnvelopes[0] == EEnvelopeType.Pan)
                {
                    DeleteEnvelopeBox((EnvelopeBox)sender);
                    if (audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopeCurves)
                        audioBlock.UpdateSplinePan(audioBlockIndex);
                    UpdatePolyLinePath();
                }
            }
            else if (Keyboard.IsKeyDown(Key.LeftCtrl) || Keyboard.IsKeyDown(Key.RightCtrl))
            {
                e.Handled = true; // Don't allow boxes to be create on top of each other.
            }
            else
            {
                this.boxDragged = true;
                boxDraggedIndex = envelopeBoxes.IndexOf((EnvelopeBox)sender);
            }
        }

        private void C_MouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            newBoxPosition = e.GetPosition(envelopesCanvas.HostCanvas);
            if (envelopesCanvas.IsVisible && boxDragged)
            {
                EnvelopeBox evb = envelopeBoxes[boxDraggedIndex];

                if (!evb.Freezed && (e.LeftButton == System.Windows.Input.MouseButtonState.Pressed))
                {
                    Point point = e.GetPosition(envelopesCanvas.HostCanvas);
                    point = UpdateEnvelopeBoxCenterPosition(evb, point, false);
                    Canvas.SetLeft(evb, point.X - evb.Width / 2);
                    Canvas.SetTop(evb, point.Y - evb.Height / 2);

                    envPolyLine.Points[boxDraggedIndex] = point;
                    UpdateEnvPoint(evb, point);
                    audioBlock.UpdateSplinePan(audioBlockIndex);
                    UpdateToolTip(evb);
                    UpdatePolyLinePath();
                    audioBlock.NotifyBuzzDataChanged();
                }
            }
        }

        bool envelopeVisible = false;
        public bool EnvelopeVisible
        {
            get
            {
                return envelopeVisible;
            }
            set
            {
                envelopeVisible = value;
                if (envelopeVisible)
                {
                    this.Visibility = Visibility.Visible;
                    envelopesCanvas.MouseRightButtonDown += HostCanvas_MouseRightButtonDown;
                }
                else
                    this.Visibility = Visibility.Hidden;
            }
        }

        MenuItem miPan;
        MenuItem menuItemPanLayerVisible;
        MenuItem miEnabled;

        public int AudioBlockIndex { get => audioBlockIndex; set => audioBlockIndex = value; }

        public MenuItem CreateEnvelopeMenu()
        {
            miPan = new MenuItem();
            miPan.Header = "Pan";
            miEnabled = new MenuItem();
            miEnabled.Header = "Enabled";
            miEnabled.IsCheckable = true;
            miEnabled.IsChecked = audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopeEnabled;
            miEnabled.Checked += MiEnabled_Checked;
            miEnabled.Unchecked += MiEnabled_Unchecked;
            miPan.Items.Add(miEnabled);
            menuItemPanLayerVisible = new MenuItem();
            menuItemPanLayerVisible.Header = "Visible";
            menuItemPanLayerVisible.IsCheckable = true;
            menuItemPanLayerVisible.IsChecked = false;
            menuItemPanLayerVisible.Checked += MenuItemPanLayerVisible_Checked;
            menuItemPanLayerVisible.Unchecked += MenuItemPanLayerVisible_Unchecked;

            miPan.Items.Add(menuItemPanLayerVisible);
            miPan.Items.Add(new Separator());
            MenuItem mi = new MenuItem();
            mi.Header = "Add Point";
            mi.Click += Mi_Click_Add_Point;
            miPan.Items.Add(mi);
            mi = new MenuItem();
            mi.Header = "Freeze All";
            mi.Click += Mi_Click_Freeze_All;
            miPan.Items.Add(mi);
            mi = new MenuItem();
            mi.Header = "Unfreeze All";
            mi.Click += Mi_Click_Unfreeze_All;
            miPan.Items.Add(mi);
            mi = new MenuItem();
            mi.Header = "Reset";
            mi.Click += Mi_Click_Reset;
            miPan.Items.Add(mi);
            mi = new MenuItem();
            mi.Header = "Curves";
            mi.IsCheckable = true;
            mi.IsChecked = audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopeCurves;
            mi.Checked += Mi_Checked_Curves;
            mi.Unchecked += Mi_Unchecked_Curves;
            miPan.Items.Add(mi);

            foreach (Control item in miPan.Items)
                if (item != menuItemPanLayerVisible && item != miEnabled)
                    item.IsEnabled = false;

            return miPan;
        }

        private void Mi_Unchecked_Curves(object sender, RoutedEventArgs e)
        {
            audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopeCurves = false;
            // ToDo: Clean cache 
            UpdatePolyLinePath();
        }

        private void Mi_Checked_Curves(object sender, RoutedEventArgs e)
        {
            audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopeCurves = true;
            audioBlock.UpdateSplinePan(audioBlockIndex);
            UpdatePolyLinePath();
        }

        private void DisableMiEvents()
        {
        }

        private void EnableMiEvents()
        {
        }

        private void Mi_Click_Reset(object sender, RoutedEventArgs e)
        {
            audioBlock.ResetPanEnvPoints(audioBlockIndex);
            this.LoadData();
            if (audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopeCurves)
                audioBlock.UpdateSplinePan(audioBlockIndex);
            this.Draw();
            audioBlock.NotifyBuzzDataChanged();
        }

        private void MiEnabled_Unchecked(object sender, RoutedEventArgs e)
        {
            audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].PanEnvelopeEnabled = false;
            audioBlock.NotifyBuzzDataChanged();
        }

        private void MiEnabled_Checked(object sender, RoutedEventArgs e)
        {
            audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].PanEnvelopeEnabled = true;
            audioBlock.NotifyBuzzDataChanged();
        }

        private void MenuItemPanLayerVisible_Unchecked(object sender, RoutedEventArgs e)
        {
            foreach (Control item in miPan.Items)
                if (item != menuItemPanLayerVisible && item != miEnabled)
                    item.IsEnabled = false;

            EnvelopeVisible = false;
        }

        private void MenuItemPanLayerVisible_Checked(object sender, RoutedEventArgs e)
        {
            foreach (Control item in miPan.Items)
                if (item != menuItemPanLayerVisible && item != miEnabled)
                    item.IsEnabled = true;

            EnvelopeVisible = true;
        }


        private void Mi_Click_Unfreeze_All(object sender, RoutedEventArgs e)
        {
            foreach (EnvelopeBox evb in envelopeBoxes)
                evb.Freezed = false;
        }

        private void Mi_Click_Freeze_All(object sender, RoutedEventArgs e)
        {
            foreach (EnvelopeBox evb in envelopeBoxes)
                evb.Freezed = true;
        }

        private void Mi_Click_Add_Point(object sender, RoutedEventArgs e)
        {
            Add_Point(newBoxPositionFromMenu);
        }

        private void Add_Point(Point pos)
        {
            EnvelopeBox evb = new EnvelopeBox(this, 10, 10, Brushes.Orange, Brushes.Black);
            evb.Freezed = false;

            evb.MouseLeftButtonDown += Evb_MouseLeftButtonDown;

            pos = UpdateEnvelopeBoxCenterPosition(evb, pos, true);
            Canvas.SetLeft(evb, pos.X - evb.Width / 2);
            Canvas.SetTop(evb, pos.Y - evb.Height / 2);

            // Update time stamp
            // ToDo: save this
            double timeStamp = ConvertPixelToTimeStampInSeconds(pos);
            double value = 0;

            if (viewMode == ViewOrientationMode.Horizontal)
            {
                value = pos.X;
            }
            else
            {
                value = pos.Y;
            }

            // Update EnvelopeLayer List
            int indexAdded = AddEnvelopeBoxToList(evb, timeStamp);

            // Update Visual
            AddEnvelopeBoxToCanvas(evb, indexAdded);

            // Update Machine Data
            SaveEnvelopeBoxData(evb, indexAdded, ScreenToValue(value), timeStamp);

            UpdateToolTip(evb);
            audioBlock.NotifyBuzzDataChanged();
        }

        private void AddEnvelopeBoxToCanvas(EnvelopeBox evb, int index)
        {
            this.Children.Add(evb);
            envPolyLine.Points.Insert(index, newBoxPosition);
        }

        private void SaveEnvelopeBoxData(EnvelopeBox evb, int index, double value, double timeStamp)
        {
            PanEnvelopePoint vep = new AudioBlock.PanEnvelopePoint();
            vep.Value = value;
            vep.TimeStamp = timeStamp;
            audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopePoints.Insert(index, vep);

            // Update spline & draw
            audioBlock.UpdateSplinePan(audioBlockIndex);
            UpdatePolyLinePath();
        }

        internal override Point MakeSureNotOverlapping(Point pos)
        {
            Point ret = pos;

            double maxTimeStamp = audioBlock.GetSampleLengthInSecondsWithOffset(audioBlockIndex);
            double posTimeStamp = ConvertPixelToTimeStampInSeconds(pos);

            List<PanEnvelopePoint> panEnvelopePoints = audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].PanEnvelopePoints;

            if (viewMode == ViewOrientationMode.Horizontal)
            {
                if (posTimeStamp > maxTimeStamp)
                {
                    posTimeStamp = maxTimeStamp;
                    ret.Y = ConvertTimeStampInSecondsToPixels(posTimeStamp);
                }

                for (int i = 0; i < panEnvelopePoints.Count; i++)
                {
                    double envTimeStamp = panEnvelopePoints[i].TimeStamp;
                    if (posTimeStamp == envTimeStamp)
                    {
                        if (i < envelopeBoxes.Count - 1)
                        {
                            double newTimeStamp = posTimeStamp + (panEnvelopePoints[i + 1].TimeStamp - posTimeStamp) / 10.0;
                            ret.Y = ConvertTimeStampInSecondsToPixels(newTimeStamp);
                        }
                        else
                        {
                            double newTimeStamp = posTimeStamp - (posTimeStamp - panEnvelopePoints[i - 1].TimeStamp) / 10.0;
                            ret.Y = ConvertTimeStampInSecondsToPixels(newTimeStamp);
                        }
                        break;
                    }
                }
            }
            else
            {
                if (posTimeStamp > maxTimeStamp)
                {
                    posTimeStamp = maxTimeStamp;
                    ret.X = ConvertTimeStampInSecondsToPixels(posTimeStamp);
                }

                for (int i = 0; i < envelopeBoxes.Count; i++)
                {
                    double envTimeStamp = panEnvelopePoints[i].TimeStamp;
                    if (posTimeStamp == envTimeStamp)
                    {
                        if (i < envelopeBoxes.Count - 1)
                        {
                            double newTimeStamp = posTimeStamp + (panEnvelopePoints[i + 1].TimeStamp - posTimeStamp) / 10.0;
                            ret.X = ConvertTimeStampInSecondsToPixels(newTimeStamp);
                        }
                        else
                        {
                            double newTimeStamp = posTimeStamp - (posTimeStamp - panEnvelopePoints[i - 1].TimeStamp) / 10.0;
                            ret.X = ConvertTimeStampInSecondsToPixels(newTimeStamp);
                        }
                        break;
                    }
                }
            }
            return ret;
        }
    }
}
