﻿using BuzzGUI.Common;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using static WDE.AudioBlock.AudioBlock;

namespace WDE.AudioBlock
{
    enum VolumeEnvelopeModes
    {
        dB0,
        dB6
    }

    /// <summary>
    /// Pan and Volume envelopes are pretty much copies with small differences.
    /// </summary>
    class EnvelopeLayerVolume : EnvelopeBase, IEnvelopeLayer
    {
        public static double ENVELOPE_MODE_0DB_MAX = 1.0;
        public static double ENVELOPE_MODE_6DB_MAX = 2.0;

        private VolumeEnvelopeModes volumeEnvelopeMode = VolumeEnvelopeModes.dB0;

        public EnvelopeLayerVolume() : base()
        {
            strokeBrush = Brushes.Black;
            fillBrush = Brushes.LightSkyBlue;
            lineBrush = new SolidColorBrush(Color.FromArgb(0x80, 0, 0, 0));
        }

        public new void Init(AudioBlock ab, Envelopes c)
        {
            base.Init(ab, c);
            envelopesCanvas.SizeChanged += EnvelopesCanvas_SizeChanged;

            VolumeEnvelopeMode = (VolumeEnvelopeModes)audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopeMode;

            envelopesCanvas.HostCanvas.MouseMove += C_MouseMove;
            envelopesCanvas.HostCanvas.MouseRightButtonDown += HostCanvas_MouseRightButtonDown;
            envelopesCanvas.HostCanvas.MouseLeave += HostCanvas_MouseLeave;
            envelopesCanvas.HostCanvas.MouseLeftButtonDown += HostCanvas_MouseLeftButtonDown;
            envelopesCanvas.HostCanvas.MouseLeftButtonUp += HostCanvas_MouseLeftButtonUp;

            LoadData();

            // Start always hidden
            EnvelopeVisible = false;
        }

        private void HostCanvas_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            this.boxDragged = false;
        }

        private void HostCanvas_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (Keyboard.IsKeyDown(Key.LeftCtrl) || Keyboard.IsKeyDown(Key.RightCtrl))
            {
                List<EEnvelopeType> visibleEnvelopes = envelopesCanvas.VisbleEnvelopes();
                if (visibleEnvelopes.Count == 1 && visibleEnvelopes[0] == EEnvelopeType.Volume)
                {
                    Add_Point(newBoxPosition);
                }
            }
        }

        internal void LoadData()
        {
            if (audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].VolumeEnvelopePoints.Count < 2)
            {
                VolumeEnvelopePoint vep = new VolumeEnvelopePoint();
                vep.Value = this.DefaulBoxValue();
                vep.TimeStamp = 0.0; // First time stamp starts always from 0 seconds.
                audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].VolumeEnvelopePoints.Add(vep);

                vep = new VolumeEnvelopePoint();
                vep.Value = this.DefaulBoxValue();
                vep.TimeStamp = audioBlock.GetSampleLengthInSecondsWithOffset(audioBlockIndex); // Last time stamp is the lenght of sample.
                audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].VolumeEnvelopePoints.Add(vep);
            }
        }

        public void UpdatePolyLinePath()
        {
            envPolyLine.Points.Clear();
            envPolyLine.Stroke = lineBrush;
            envPolyLine.StrokeThickness = 2;

            if (viewMode == ViewOrientationMode.Horizontal)
            {
                if (!audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopeCurves)
                {
                    foreach (EnvelopeBox evb in envelopeBoxes)
                    {
                        Point point = new Point(ValueToScreen(GetEnvelopeValue(evb)), ConvertTimeStampInSecondsToPixels(GetEnvelopeTimeStamp(evb)));
                        point = UpdateEnvelopeBoxCenterPosition(evb, point, false);
                        envPolyLine.Points.Add(point);
                    }
                }
                else
                {
                    SplineCache spline = audioBlock.GetSplineVol(audioBlockIndex);
                    double slidingOffsetWindowInSeconds = envelopesCanvas.HostCanvas.SlidingWindowOffsetSeconds;

                    for (int i = 0; i < spline.YValues.Length; i++)
                    {
                        double y = this.Height * (((double)i * spline.StepSizeInSeconds - slidingOffsetWindowInSeconds) / DrawLengthInSeconds);
                        double x = this.Width * ((spline.YValues[i] / MaxBoxValue()) * ENVELOPE_VIEW_SCALE_ADJUST + ((1.0 - ENVELOPE_VIEW_SCALE_ADJUST) / 2.0));
                        Point polyPoint = new Point(x, y);
                        envPolyLine.Points.Add(polyPoint);
                    }
                }
            }
            else
            {
                if (!audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopeCurves)
                {
                    foreach (EnvelopeBox evb in envelopeBoxes)
                    {
                        Point point = new Point(ConvertTimeStampInSecondsToPixels(GetEnvelopeTimeStamp(evb)), ValueToScreen(GetEnvelopeValue(evb)));
                        point = UpdateEnvelopeBoxCenterPosition(evb, point, false);
                        envPolyLine.Points.Add(point);
                    }
                }
                else
                {
                    //audioBlock.UpdateSplineVol(audioBlockIndex);
                    SplineCache spline = audioBlock.GetSplineVol(audioBlockIndex);
                    double slidingOffsetWindowInSeconds = envelopesCanvas.HostCanvas.SlidingWindowOffsetSeconds;

                    for (int i = 0; i < spline.YValues.Length; i++)
                    {
                        double x = this.Width * (((double)i * spline.StepSizeInSeconds - slidingOffsetWindowInSeconds) / DrawLengthInSeconds);
                        double y = this.Height * ((1.0 - (spline.YValues[i] / MaxBoxValue())) * ENVELOPE_VIEW_SCALE_ADJUST + ((1.0 - ENVELOPE_VIEW_SCALE_ADJUST) / 2.0));
                        Point polyPoint = new Point(x, y);
                        envPolyLine.Points.Add(polyPoint);
                    }
                }
            }
        }

        public void Draw()
        {
            this.Children.Clear();
            envelopeBoxes.Clear();
            this.Children.Add(envPolyLine);

            if (viewMode == ViewOrientationMode.Horizontal)
            {
                foreach (VolumeEnvelopePoint vep in audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].VolumeEnvelopePoints)
                {
                    EnvelopeBox env = new EnvelopeBox(this, 10, 10, fillBrush, strokeBrush);
                    envelopeBoxes.Add(env);
                    env.MouseLeftButtonDown += Evb_MouseLeftButtonDown;

                    Point point = new Point(ValueToScreen(vep.Value), ConvertTimeStampInSecondsToPixels(vep.TimeStamp));
                    point = UpdateEnvelopeBoxCenterPosition(env, point, false);

                    Canvas.SetLeft(env, point.X - env.Width / 2);
                    Canvas.SetTop(env, point.Y - env.Height / 2);

                    env.ToolTip = string.Format("Time: {0:0.0}, Volume: {1:0.0} bB", vep.TimeStamp, Decibel.FromAmplitude(vep.Value));

                    this.Children.Add(env);
                }
            }
            else
            {
                foreach (VolumeEnvelopePoint vep in audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].VolumeEnvelopePoints)
                {
                    EnvelopeBox env = new EnvelopeBox(this, 10, 10, fillBrush, strokeBrush);
                    envelopeBoxes.Add(env);
                    env.MouseLeftButtonDown += Evb_MouseLeftButtonDown;
                    // env.MouseLeftButtonUp += Evb_MouseLeftButtonUp;


                    Point point = new Point(ConvertTimeStampInSecondsToPixels(vep.TimeStamp), ValueToScreen(vep.Value));
                    point = UpdateEnvelopeBoxCenterPosition(env, point, false);

                    Canvas.SetLeft(env, point.X - env.Width / 2);
                    Canvas.SetTop(env, point.Y - env.Height / 2);

                    env.ToolTip = string.Format("Time: {0:0.0}, Volume: {1:0.0} bB", vep.TimeStamp, Decibel.FromAmplitude(vep.Value));

                    this.Children.Add(env);
                }
            }

            UpdatePolyLinePath();
        }

        public void UpdateToolTip(EnvelopeBox env)
        {
            env.ToolTip = string.Format("Time: {0:0.00} Seconds, Volume: {1:0.0} bB", GetEnvelopeTimeStamp(env), Decibel.FromAmplitude(GetEnvelopeValue(env)));
        }

        public void DeleteEnvelopeBox(EnvelopeBox envelopeBox)
        {
            // Can't delete first or last
            int envelopeBoxIndex = envelopeBoxes.IndexOf(envelopeBox);
            if ((envelopeBoxIndex > 0) && (envelopeBoxIndex < envelopeBoxes.Count - 1))
            {
                envPolyLine.Points.RemoveAt(envelopeBoxIndex);
                envelopeBoxes.Remove(envelopeBox);
                this.Children.Remove(envelopeBox);

                audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopePoints.RemoveAt(envelopeBoxIndex);
                if (audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopeCurves)
                    audioBlock.UpdateSplineVol(audioBlockIndex);
                UpdatePolyLinePath();
            }
        }

        internal override double GetEnvelopeValue(EnvelopeBox envelopeBox)
        {
            int envelopeBoxIndex = envelopeBoxes.IndexOf(envelopeBox);
            return GetEnvelopeValue(envelopeBoxIndex);
        }

        internal override double GetEnvelopeValue(int envelopeBoxIndex)
        {
            return audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopePoints[envelopeBoxIndex].Value;
        }

        internal override double GetEnvelopeTimeStamp(EnvelopeBox envelopeBox)
        {
            int envelopeBoxIndex = envelopeBoxes.IndexOf(envelopeBox);
            return GetEnvelopeTimeStamp(envelopeBoxIndex);
        }

        internal override double GetEnvelopeTimeStamp(int envelopeBoxIndex)
        {
            return audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopePoints[envelopeBoxIndex].TimeStamp;
        }

        internal override void SetEnvelopeValue(EnvelopeBox envelopeBox, double value)
        {
            int envelopeBoxIndex = envelopeBoxes.IndexOf(envelopeBox);
            SetEnvelopeValue(envelopeBoxIndex, value);
        }

        internal override void SetEnvelopeValue(int envelopeBoxIndex, double value)
        {
            audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopePoints[envelopeBoxIndex].Value = value;
        }

        internal override void SetEnvelopeTimeStamp(EnvelopeBox envelopeBox, double value)
        {
            int envelopeBoxIndex = envelopeBoxes.IndexOf(envelopeBox);
            SetEnvelopeTimeStamp(envelopeBoxIndex, value);
        }

        internal override void SetEnvelopeTimeStamp(int envelopeBoxIndex, double value)
        {
            audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopePoints[envelopeBoxIndex].TimeStamp = value;
        }

        public new void ResetEnvelopeBox(EnvelopeBox envelopeBox)
        {
            base.ResetEnvelopeBox(envelopeBox);

            if (audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopeCurves)
                audioBlock.UpdateSplineVol(audioBlockIndex);
            UpdatePolyLinePath();
        }

        private void EnvelopesCanvas_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            this.Width = ((Canvas)sender).Width;
            this.Height = ((Canvas)sender).Height;
            Draw();
        }

        internal override double DefaulBoxValue()
        {
            if (VolumeEnvelopeMode == VolumeEnvelopeModes.dB0)
            {
                return ENVELOPE_MODE_0DB_MAX;
            }
            else
            {
                return ENVELOPE_MODE_0DB_MAX;
            }
        }

        internal override double MaxBoxValue()
        {
            if (VolumeEnvelopeMode == VolumeEnvelopeModes.dB0)
            {
                return ENVELOPE_MODE_0DB_MAX;
            }
            else
            {
                return ENVELOPE_MODE_6DB_MAX;
            }
        }

        private void HostCanvas_MouseLeave(object sender, MouseEventArgs e)
        {
            this.boxDragged = false;
        }

        private void HostCanvas_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            newBoxPositionFromMenu = e.GetPosition(envelopesCanvas.HostCanvas);
        }

        private void Evb_MouseLeftButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if ((Keyboard.IsKeyDown(Key.LeftShift) || Keyboard.IsKeyDown(Key.RightShift)) && !((EnvelopeBox)sender).Freezed)
            {
                List<EEnvelopeType> visibleEnvelopes = envelopesCanvas.VisbleEnvelopes();
                if (visibleEnvelopes.Count == 1 && visibleEnvelopes[0] == EEnvelopeType.Volume)
                {
                    DeleteEnvelopeBox((EnvelopeBox)sender);
                    if (audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopeCurves)
                        audioBlock.UpdateSplineVol(audioBlockIndex);
                    UpdatePolyLinePath();
                }
            }
            else if (Keyboard.IsKeyDown(Key.LeftCtrl) || Keyboard.IsKeyDown(Key.RightCtrl))
            {
                e.Handled = true; // Don't allow boxes to be create on top of each other.
            }
            else
            {
                this.boxDragged = true;
                boxDraggedIndex = envelopeBoxes.IndexOf((EnvelopeBox)sender);
            }
        }

        private void C_MouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            newBoxPosition = e.GetPosition(envelopesCanvas.HostCanvas);
            if (envelopesCanvas.IsVisible && boxDragged)
            {
                EnvelopeBox evb = envelopeBoxes[boxDraggedIndex];

                if (!evb.Freezed && (e.LeftButton == System.Windows.Input.MouseButtonState.Pressed))
                {
                    Point point = e.GetPosition(envelopesCanvas.HostCanvas);
                    point = UpdateEnvelopeBoxCenterPosition(evb, point, false);
                    Canvas.SetLeft(evb, point.X - evb.Width / 2);
                    Canvas.SetTop(evb, point.Y - evb.Height / 2);

                    // envPolyLine.Points[boxDraggedIndex] = point;                    
                    UpdateEnvPoint(evb, point);
                    audioBlock.UpdateSplineVol(audioBlockIndex);
                    UpdateToolTip(evb);
                    UpdatePolyLinePath();
                    audioBlock.NotifyBuzzDataChanged();
                }
            }
        }

        bool envelopeVisible = false;
        public bool EnvelopeVisible
        {
            get
            {
                return envelopeVisible;
            }
            set
            {
                envelopeVisible = value;
                if (envelopeVisible)
                {
                    this.Visibility = Visibility.Visible;
                    this.MouseRightButtonDown += HostCanvas_MouseRightButtonDown;
                }
                else
                    this.Visibility = Visibility.Hidden;
            }
        }

        MenuItem miVolume;
        MenuItem menuItemVolumeLayerVisible;
        MenuItem miVolume0dB;
        MenuItem miVolume6dB;
        MenuItem miEnabled;
        private Point newBoxPosition;

        public int AudioBlockIndex { get => audioBlockIndex; set => audioBlockIndex = value; }

        internal VolumeEnvelopeModes VolumeEnvelopeMode { get => volumeEnvelopeMode; set => volumeEnvelopeMode = value; }

        public MenuItem CreateEnvelopeMenu()
        {
            miVolume = new MenuItem();
            miVolume.Header = "Volume";
            miEnabled = new MenuItem();
            miEnabled.Header = "Enabled";
            miEnabled.IsCheckable = true;
            miEnabled.IsChecked = audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopeEnabled;
            miEnabled.Checked += MiEnabled_Checked;
            miEnabled.Unchecked += MiEnabled_Unchecked;
            miVolume.Items.Add(miEnabled);
            menuItemVolumeLayerVisible = new MenuItem();
            menuItemVolumeLayerVisible.Header = "Visible";
            menuItemVolumeLayerVisible.IsCheckable = true;
            menuItemVolumeLayerVisible.IsChecked = false;
            menuItemVolumeLayerVisible.Checked += MenuItemVolumeLayerVisible_Checked;
            menuItemVolumeLayerVisible.Unchecked += MenuItemVolumeLayerVisible_Unchecked;

            miVolume.Items.Add(menuItemVolumeLayerVisible);
            miVolume.Items.Add(new Separator());
            MenuItem mi = new MenuItem();
            mi.Header = "Add Point";
            mi.Click += Mi_Click_Add_Point;
            miVolume.Items.Add(mi);
            mi = new MenuItem();
            mi.Header = "Freeze All";
            mi.Click += Mi_Click_Freeze_All;
            miVolume.Items.Add(mi);
            mi = new MenuItem();
            mi.Header = "Unfreeze All";
            mi.Click += Mi_Click_Unfreeze_All;
            miVolume.Items.Add(mi);
            mi = new MenuItem();
            mi.Header = "Reset";
            mi.Click += Mi_Click_Reset;
            miVolume.Items.Add(mi);
            MenuItem miMode = new MenuItem();
            miMode.Header = "Mode";
            miVolume.Items.Add(miMode);
            miVolume0dB = new MenuItem();
            miVolume0dB.IsCheckable = true;
            miVolume0dB.IsChecked = VolumeEnvelopeMode == VolumeEnvelopeModes.dB0;
            miVolume0dB.Checked += Mi_Checked_0dB;
            miVolume0dB.Unchecked += MiVolume0dB_Unchecked;
            miVolume0dB.Header = string.Format("Max {0:0.0} bB", Decibel.FromAmplitude(1.0));
            miMode.Items.Add(miVolume0dB);
            miVolume6dB = new MenuItem();
            miVolume6dB.IsCheckable = true;
            miVolume6dB.IsChecked = VolumeEnvelopeMode == VolumeEnvelopeModes.dB6;
            miVolume6dB.Checked += Mi_Checked_6dB;
            miVolume6dB.Unchecked += MiVolume6dB_Unchecked;
            miVolume6dB.Header = string.Format("Max {0:0.0} bB", Decibel.FromAmplitude(2.0));
            miMode.Items.Add(miVolume6dB);
            mi = new MenuItem();
            mi.Header = "Curves";
            mi.IsCheckable = true;
            mi.IsChecked = audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopeCurves;
            mi.Checked += Mi_Checked_Curves;
            mi.Unchecked += Mi_Unchecked_Curves;
            miVolume.Items.Add(mi);

            foreach (Control item in miVolume.Items)
                if (item != menuItemVolumeLayerVisible && item != miEnabled)
                    item.IsEnabled = false;

            return miVolume;
        }

        private void Mi_Unchecked_Curves(object sender, RoutedEventArgs e)
        {
            audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopeCurves = false;
            // ToDo: Clean cache 
            UpdatePolyLinePath();
            audioBlock.NotifyBuzzDataChanged();
        }

        private void Mi_Checked_Curves(object sender, RoutedEventArgs e)
        {
            audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopeCurves = true;
            audioBlock.UpdateSplineVol(audioBlockIndex);
            UpdatePolyLinePath();
            audioBlock.NotifyBuzzDataChanged();
        }

        private void DisableMiEvents()
        {
            miVolume6dB.Checked -= Mi_Checked_6dB;
            miVolume6dB.Unchecked -= MiVolume6dB_Unchecked;
            miVolume0dB.Checked -= Mi_Checked_0dB;
            miVolume0dB.Unchecked -= MiVolume0dB_Unchecked;
        }

        private void EnableMiEvents()
        {
            miVolume6dB.Checked += Mi_Checked_6dB;
            miVolume6dB.Unchecked += MiVolume6dB_Unchecked;
            miVolume0dB.Checked += Mi_Checked_0dB;
            miVolume0dB.Unchecked += MiVolume0dB_Unchecked;
        }

        private void MiVolume6dB_Unchecked(object sender, RoutedEventArgs e)
        {
            // User can't unclick checkbox.
            DisableMiEvents();
            miVolume6dB.IsChecked = true;
            EnableMiEvents();
        }

        private void MiVolume0dB_Unchecked(object sender, RoutedEventArgs e)
        {
            DisableMiEvents();
            miVolume0dB.IsChecked = true;
            EnableMiEvents();
        }

        private void Mi_Click_Reset(object sender, RoutedEventArgs e)
        {
            audioBlock.ResetVolEnvPoints(AudioBlockIndex);
            this.LoadData();
            if (audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopeCurves)
                audioBlock.UpdateSplineVol(audioBlockIndex);
            this.Draw();
            audioBlock.NotifyBuzzDataChanged();
        }

        private void MiEnabled_Unchecked(object sender, RoutedEventArgs e)
        {
            audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].VolumeEnvelopeEnabled = false;
            audioBlock.NotifyBuzzDataChanged();
        }

        private void MiEnabled_Checked(object sender, RoutedEventArgs e)
        {
            audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].VolumeEnvelopeEnabled = true;
            audioBlock.NotifyBuzzDataChanged();
        }

        private void MenuItemVolumeLayerVisible_Unchecked(object sender, RoutedEventArgs e)
        {
            foreach (Control item in miVolume.Items)
                if (item != menuItemVolumeLayerVisible && item != miEnabled)
                    item.IsEnabled = false;

            EnvelopeVisible = false;
        }

        private void MenuItemVolumeLayerVisible_Checked(object sender, RoutedEventArgs e)
        {
            foreach (Control item in miVolume.Items)
                if (item != menuItemVolumeLayerVisible && item != miEnabled)
                    item.IsEnabled = true;

            EnvelopeVisible = true;
        }

        private void Mi_Checked_6dB(object sender, RoutedEventArgs e)
        {
            VolumeEnvelopeMode = VolumeEnvelopeModes.dB6;
            audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].VolumeEnvelopeMode = (int)VolumeEnvelopeMode;
            DisableMiEvents();
            miVolume0dB.IsChecked = false;
            EnableMiEvents();

            Draw();

            audioBlock.NotifyBuzzDataChanged();
        }

        private void Mi_Checked_0dB(object sender, RoutedEventArgs e)
        {
            VolumeEnvelopeMode = VolumeEnvelopeModes.dB0;
            audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].VolumeEnvelopeMode = (int)VolumeEnvelopeMode;
            DisableMiEvents();
            miVolume6dB.IsChecked = false;
            EnableMiEvents();

            // Cut all down to 0dB
            foreach (VolumeEnvelopePoint vep in audioBlock.MachineState.AudioBlockInfoTable[AudioBlockIndex].VolumeEnvelopePoints)
                if (vep.Value > ENVELOPE_MODE_0DB_MAX)
                    vep.Value = ENVELOPE_MODE_0DB_MAX;

            Draw();

            audioBlock.NotifyBuzzDataChanged();
        }

        private void Mi_Click_Unfreeze_All(object sender, RoutedEventArgs e)
        {
            foreach (EnvelopeBox evb in envelopeBoxes)
                evb.Freezed = false;
        }

        private void Mi_Click_Freeze_All(object sender, RoutedEventArgs e)
        {
            foreach (EnvelopeBox evb in envelopeBoxes)
                evb.Freezed = true;
        }

        private void Mi_Click_Add_Point(object sender, RoutedEventArgs e)
        {
            Add_Point(newBoxPositionFromMenu);
        }

        private void Add_Point(Point pos)
        {
            EnvelopeBox evb = new EnvelopeBox(this, 10, 10, Brushes.LightSkyBlue, Brushes.Black);
            evb.Freezed = false;

            evb.MouseLeftButtonDown += Evb_MouseLeftButtonDown;

            pos = UpdateEnvelopeBoxCenterPosition(evb, pos, true);
            Canvas.SetLeft(evb, pos.X - evb.Width / 2);
            Canvas.SetTop(evb, pos.Y - evb.Height / 2);

            // Update time stamp
            // ToDo: save this
            double timeStamp = ConvertPixelToTimeStampInSeconds(pos);
            double value = 0;

            if (viewMode == ViewOrientationMode.Horizontal)
            {
                value = pos.X;
            }
            else
            {
                value = pos.Y;
            }

            // Update EnvelopeLayer List
            int indexAdded = AddEnvelopeBoxToList(evb, timeStamp);

            // Update Visual
            AddEnvelopeBoxToCanvas(evb, indexAdded);

            // Update Machine Data
            SaveEnvelopeBoxData(evb, indexAdded, ScreenToValue(value), timeStamp);

            UpdateToolTip(evb);
            audioBlock.NotifyBuzzDataChanged();
        }

        private void AddEnvelopeBoxToCanvas(EnvelopeBox evb, int index)
        {
            this.Children.Add(evb);
            envPolyLine.Points.Insert(index, newBoxPositionFromMenu);
        }

        private void SaveEnvelopeBoxData(EnvelopeBox evb, int index, double value, double timeStamp)
        {
            AudioBlock.VolumeEnvelopePoint vep = new AudioBlock.VolumeEnvelopePoint();
            vep.Value = value;
            vep.TimeStamp = timeStamp;
            audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopePoints.Insert(index, vep);

            // Update spline & draw
            audioBlock.UpdateSplineVol(audioBlockIndex);
            UpdatePolyLinePath();
        }

        internal override Point MakeSureNotOverlapping(Point pos)
        {
            Point ret = pos;

            double maxTimeStamp = audioBlock.GetSampleLengthInSecondsWithOffset(audioBlockIndex);
            double posTimeStamp = ConvertPixelToTimeStampInSeconds(pos);

            List<VolumeEnvelopePoint> volumeEnvelopePoints = audioBlock.MachineState.AudioBlockInfoTable[audioBlockIndex].VolumeEnvelopePoints;

            if (viewMode == ViewOrientationMode.Horizontal)
            {
                if (posTimeStamp > maxTimeStamp)
                {
                    posTimeStamp = maxTimeStamp;
                    ret.Y = ConvertTimeStampInSecondsToPixels(posTimeStamp);
                }

                for (int i = 0; i < volumeEnvelopePoints.Count; i++)
                {
                    double envTimeStamp = volumeEnvelopePoints[i].TimeStamp;
                    if (posTimeStamp == envTimeStamp)
                    {
                        if (i < envelopeBoxes.Count - 1)
                        {
                            double newTimeStamp = posTimeStamp + (volumeEnvelopePoints[i + 1].TimeStamp - posTimeStamp) / 10.0;
                            ret.Y = ConvertTimeStampInSecondsToPixels(newTimeStamp);
                        }
                        else
                        {
                            double newTimeStamp = posTimeStamp - (posTimeStamp - volumeEnvelopePoints[i - 1].TimeStamp) / 10.0;
                            ret.Y = ConvertTimeStampInSecondsToPixels(newTimeStamp);
                        }
                        break;
                    }
                }
            }
            else
            {
                if (posTimeStamp > maxTimeStamp)
                {
                    posTimeStamp = maxTimeStamp;
                    ret.X = ConvertTimeStampInSecondsToPixels(posTimeStamp);
                }

                for (int i = 0; i < envelopeBoxes.Count; i++)
                {
                    double envTimeStamp = volumeEnvelopePoints[i].TimeStamp;
                    if (posTimeStamp == envTimeStamp)
                    {
                        if (i < envelopeBoxes.Count - 1)
                        {
                            double newTimeStamp = posTimeStamp + (volumeEnvelopePoints[i + 1].TimeStamp - posTimeStamp) / 10.0;
                            ret.X = ConvertTimeStampInSecondsToPixels(newTimeStamp);
                        }
                        else
                        {
                            double newTimeStamp = posTimeStamp - (posTimeStamp - volumeEnvelopePoints[i - 1].TimeStamp) / 10.0;
                            ret.X = ConvertTimeStampInSecondsToPixels(newTimeStamp);
                        }
                        break;
                    }
                }
            }
            return ret;
        }
    }
}
