﻿using System.Collections.Generic;
using System.Windows.Controls;
using static WDE.AudioBlock.AudioBlock;

namespace WDE.AudioBlock
{
    /// <summary>
    /// Hosts all envelopes for Wave Canvas instance.
    /// </summary>
    class Envelopes : Canvas
    {
        private ViewOrientationMode viewOrientationMode = ViewOrientationMode.Horizontal;
        private EnvelopeLayerVolume layerVolume;
        private EnvelopeLayerPan layerPan;
        private AudioBlock audioBlock;
        private WaveCanvas hostCanvas;
        private double drawLengthInSeconds;
        public Envelopes(AudioBlock ab, WaveCanvas parentCanvas)
        {
            this.audioBlock = ab;

            this.Width = parentCanvas.Width;
            this.Height = parentCanvas.Height;

            Canvas.SetLeft(this, 0);
            Canvas.SetTop(this, 0);

            HostCanvas = parentCanvas;
            HostCanvas.SizeChanged += ParentCanvas_SizeChanged;

            layerVolume = new EnvelopeLayerVolume();
            layerVolume.Init(audioBlock, this);
            layerVolume.LoadData();
            this.Children.Add(layerVolume);

            parentCanvas.ContextMenu.Items.Insert(0, layerVolume.CreateEnvelopeMenu());

            layerPan = new EnvelopeLayerPan();
            layerPan.Init(audioBlock, this);
            layerPan.LoadData();
            this.Children.Add(layerPan);

            parentCanvas.ContextMenu.Items.Insert(1, layerPan.CreateEnvelopeMenu());
        }

        private void ParentCanvas_SizeChanged(object sender, System.Windows.SizeChangedEventArgs e)
        {
            this.Width = ((Canvas)sender).Width;
            this.Height = ((Canvas)sender).Height;
            layerVolume.Draw();
            layerPan.Draw();
        }

        internal ViewOrientationMode ViewOrientationMode
        {
            get
            {
                return viewOrientationMode;
            }
            set
            {
                viewOrientationMode = value;

                layerVolume.SetOrientation(viewOrientationMode);
                layerPan.SetOrientation(viewOrientationMode);
            }
        }

        public void Draw()
        {
            layerVolume.Draw();
            layerVolume.UpdatePolyLinePath();
            layerPan.Draw();
            layerPan.UpdatePolyLinePath();
        }

        public WaveCanvas HostCanvas { get => hostCanvas; set => hostCanvas = value; }
        public double DrawLengthInSeconds
        {
            get => drawLengthInSeconds;
            set
            {
                drawLengthInSeconds = value;
                layerVolume.DrawLengthInSeconds = drawLengthInSeconds;
                layerPan.DrawLengthInSeconds = drawLengthInSeconds;
            }
        }

        public List<EEnvelopeType> VisbleEnvelopes()
        {
            List<EEnvelopeType> visibleEnvelopes = new List<EEnvelopeType>();
            if (layerVolume.EnvelopeVisible)
            {
                EEnvelopeType vol = EEnvelopeType.Volume;
                visibleEnvelopes.Add(vol);
            }
            if (layerPan.EnvelopeVisible)
            {
                EEnvelopeType pan = EEnvelopeType.Pan;
                visibleEnvelopes.Add(pan);
            }
            return visibleEnvelopes;
        }
    }
}
